package cc.fascinated.bat.command.impl.botadmin.premium;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "set", description = "Adds premium to a guild")
public class SetSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public SetSubCommand(GuildService guildService) {
        this.guildService = guildService;
        super.addOption(OptionType.STRING, "guild", "The guild id to set as premium", true);
        super.addOption(OptionType.BOOLEAN, "infinite", "Whether the premium length should be infinite", true);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        OptionMapping guildOption = interaction.getOption("guild");
        if (guildOption == null) {
            interaction.reply("Please provide a guild id").queue();
            return;
        }
        String guildId = guildOption.getAsString();
        OptionMapping infiniteOption = interaction.getOption("infinite");
        if (infiniteOption == null) {
            interaction.reply("Please provide whether the premium length should be infinite").queue();
            return;
        }

        boolean infinite = infiniteOption.getAsBoolean();
        BatGuild batGuild = guildService.getGuild(guildId);
        if (batGuild == null) {
            interaction.reply("The guild with the id %s does not exist".formatted(guildId)).queue();
            return;
        }
        BatGuild.Premium premium = batGuild.getPremium();
        if (!infinite) {
            premium.addTime();
        } else {
            premium.addInfiniteTime();
        }
        guildService.saveGuild(batGuild);
        if (!infinite) {
            interaction.reply("The guild **%s** has been set as premium until <t:%s>".formatted(guild.getName(), premium.getExpiresAt().toInstant().toEpochMilli() / 1000)).queue();
        } else {
            interaction.reply("The guild **%s** has been set as premium indefinitely".formatted(guild.getName())).queue();
        }
    }
}
