package cc.fascinated.bat.features.afk.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.MemberUtils;
import cc.fascinated.bat.features.afk.profile.AfkProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "afk", description = "Sets your AFK status")
public class AfkCommand extends BatCommand {
    private final GuildService guildService;

    @Autowired
    public AfkCommand(@NonNull GuildService guildService) {
        this.guildService = guildService;
        super.addOption(OptionType.STRING, "reason", "The reason for being AFK", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        AfkProfile profile = guild.getProfile(AfkProfile.class);
        String reason = null;
        OptionMapping reasonOption = interaction.getOption("reason");
        if (reasonOption != null) {
            reason = reasonOption.getAsString();
        }

        profile.addAfkUser(guild, member.getId(), reason);
        guildService.saveGuild(guild);
        interaction.reply("You are now AFK: %s%s".formatted(
                profile.getAfkReason(member.getId()),
                MemberUtils.hasPermissionToEdit(guild, user) ? "" :
                        "\n\n*I do not have enough permissions to edit your user, and therefore cannot update your nickname*"
        )).queue();
    }
}
