package cc.fascinated.bat.features.autorole.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("autoroles:remove.sub")
@CommandInfo(name = "remove", description = "Removes a role from the auto roles list")
public class RemoveSubCommand extends BatSubCommand {
    private final GuildService guildService;

    @Autowired
    public RemoveSubCommand(GuildService guildService) {
        super.addOption(OptionType.ROLE, "role", "The role to remove", true);
        this.guildService = guildService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        OptionMapping option = interaction.getOption("role");
        if (option == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("Please provide a role to remove")
                    .build()).queue();
            return;
        }

        Role role = option.getAsRole();
        if (!profile.hasRole(role.getId())) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The role %s is not in the auto roles list".formatted(role.getAsMention()))
                    .build()).queue();
            return;
        }

        profile.removeRole(role.getId());
        guildService.saveGuild(guild);
        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully removed the role %s from the auto roles list".formatted(role.getAsMention()))
                .build()).queue();
    }
}
