package cc.fascinated.bat.features.scoresaber;

import cc.fascinated.bat.common.NumberUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.scoresaber.profile.GuildNumberOneScoreFeedProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.GuildService;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
public class NumberOneScoreFeedListener implements EventListener {
    private final GuildService guildService;

    @Autowired
    public NumberOneScoreFeedListener(GuildService guildService) {
        this.guildService = guildService;
    }

    @Override
    public void onScoresaberScoreReceived(@NotNull ScoreSaberPlayerScoreToken score, @NotNull ScoreSaberLeaderboardToken leaderboard,
                                          @NotNull ScoreSaberScoreToken.LeaderboardPlayerInfo player) {
        if (score.getScore().getRank() != 1) { // Only send if the score is a #1 score
            return;
        }
        if (!leaderboard.isRanked()) { // Only send if the leaderboard is ranked
            return;
        }
        log.info("A new #1 score has been set by {} on {} ({})!",
                player.getName(),
                leaderboard.getSongName(),
                "%s⭐".formatted(NumberUtils.formatNumberCommas(leaderboard.getStars()))
        );

        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) {
                continue;
            }
            GuildNumberOneScoreFeedProfile profile = batGuild.getProfile(GuildNumberOneScoreFeedProfile.class);
            if (profile == null || profile.getChannelId() == null) {
                continue;
            }

            TextChannel channel = profile.getAsTextChannel();
            if (channel == null) {
                log.error("Scoresaber user feed channel is null for guild {}, removing the stored channel.", guild.getId());
                profile.setChannelId(null);
                guildService.saveGuild(batGuild);
                continue;
            }
            channel.sendMessageEmbeds(ScoreSaberFeature.buildScoreEmbed(score)).queue();
        }
    }
}
