package cc.fascinated.bat.features.scoresaber.command.userfeed;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.scoresaber.profile.GuildUserScoreFeedProfile;
import cc.fascinated.bat.features.scoresaber.profile.UserScoreSaberProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.GuildService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("scoresaber-user-feed:user.sub")
@CommandInfo(name = "user", description = "Adds or removes a user from the feed")
public class UserSubCommand extends BatSubCommand {
    private final GuildService guildService;
    private final UserService userService;

    @Autowired
    public UserSubCommand(GuildService guildService, UserService userService) {
        super.addOption(OptionType.USER, "user", "Add or remove a user from the score feed", false);
        this.guildService = guildService;
        this.userService = userService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        GuildUserScoreFeedProfile profile = guild.getProfile(GuildUserScoreFeedProfile.class);
        OptionMapping option = interaction.getOption("user");
        if (option == null) {
            if (profile.getTrackedUsers().isEmpty()) {
                interaction.replyEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("There are no users being tracked in the feed")
                        .build()).queue();
                return;
            }
            StringBuilder stringBuilder = new StringBuilder();
            for (String accountId : profile.getTrackedUsers()) {
                stringBuilder.append("[%s](%s)".formatted(
                        accountId,
                        "https://scoresaber.com/u/%s".formatted(accountId)
                ));
            }
            interaction.replyEmbeds(EmbedUtils.genericEmbed()
                    .setDescription("The current users being tracked in the feed are:\n%s".formatted(stringBuilder.toString()))
                    .build()).queue();
            return;
        }

        User target = option.getAsUser();
        BatUser targetUser = userService.getUser(target.getId());
        UserScoreSaberProfile targetProfile = targetUser.getProfile(UserScoreSaberProfile.class);
        if (targetProfile.getSteamId() == null) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The user you are trying to track does not have a linked ScoreSaber profile")
                    .build()).queue();
            return;
        }

        if (profile.getTrackedUsers().contains(targetProfile.getSteamId())) {
            profile.getTrackedUsers().remove(targetProfile.getSteamId());
            interaction.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription("Successfully removed %s from the feed".formatted(target.getAsMention()))
                    .build()).queue();
        } else {
            profile.getTrackedUsers().add(targetProfile.getSteamId());
            interaction.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription("Successfully added %s to the feed".formatted(target.getAsMention()))
                    .build()).queue();
        }
        guildService.saveGuild(guild);
    }
}
