package cc.fascinated.bat.features.scoresaber.profile;

import cc.fascinated.bat.common.Profile;
import cc.fascinated.bat.service.DiscordService;
import lombok.Getter;
import lombok.Setter;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
@Setter
public class GuildUserScoreFeedProfile extends Profile {
    /**
     * The channel ID of the score feed
     */
    private String channelId;

    /**
     * The users that are being tracked
     */
    private List<String> trackedUsers;

    public GuildUserScoreFeedProfile() {
        super("scoresaber-user-score-feed");
    }

    /**
     * Gets the tracked users
     *
     * @return the tracked users
     */
    public List<String> getTrackedUsers() {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        return this.trackedUsers;
    }

    /**
     * Adds a user to be tracked
     *
     * @param userId the user ID to add
     */
    public void addTrackedUser(String userId) {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        trackedUsers.add(userId);
    }

    /**
     * Removes a user from being tracked
     *
     * @param userId the user ID to remove
     */
    public void removeTrackedUser(String userId) {
        if (this.trackedUsers == null) {
            this.trackedUsers = new ArrayList<>();
        }
        trackedUsers.remove(userId);
    }

    /**
     * Gets the channel as a TextChannel
     *
     * @return the channel as a TextChannel
     */
    public TextChannel getAsTextChannel() {
        return DiscordService.JDA.getTextChannelById(channelId);
    }

    @Override
    public void reset() {
        this.channelId = null;
        this.trackedUsers = null;
    }
}
